@@ -20,7 +20,7 @@ class _2chanThreadExtractor(Extractor):
     filename_fmt = "{tim}.{extension}"
     archive_fmt = "{board}_{thread}_{tim}"
     url_fmt = "https://{server}.2chan.net/{board}/src/{filename}"
-    pattern = r"(?:https?://)?([^.]+)\.2chan\.net/([^/]+)/res/(\d+)"
+    pattern = r"(?:https?://)?([\w-]+)\.2chan\.net/([^/]+)/res/(\d+)"
     test = ("http://dec.2chan.net/70/res/4752.htm", {
         "url": "f49aa31340e9a3429226af24e19e01f5b819ca1f",
         "keyword": "44599c21b248e79692b2eb2da12699bd0ed5640a",
@@ -35,7 +35,6 @@ class _2chanThreadExtractor(Extractor):
             self.server, self.board, self.thread)
         page = self.request(url).text
         data = self.metadata(page)
-        yield Message.Version, 1
         yield Message.Directory, data
         for post in self.posts(page):
             if "filename" not in post:
