@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -31,7 +31,6 @@ modules = [
     "desktopography",
     "deviantart",
     "dynastyscans",
-    "e621",
     "erome",
     "exhentai",
     "fallenangels",
@@ -44,6 +43,7 @@ modules = [
     "gelbooru_v01",
     "gelbooru_v02",
     "gfycat",
+    "gofile",
     "hbrowse",
     "hentai2read",
     "hentaicosplays",
@@ -68,7 +68,10 @@ modules = [
     "keenspot",
     "kemonoparty",
     "khinsider",
+    "kissgoddess",
+    "kohlchan",
     "komikcast",
+    "lightroom",
     "lineblog",
     "livedoor",
     "luscious",
@@ -80,6 +83,7 @@ modules = [
     "mangapark",
     "mangasee",
     "mangoxo",
+    "mememuseum",
     "myhentaigallery",
     "myportfolio",
     "naver",
@@ -95,6 +99,7 @@ modules = [
     "philomena",
     "photobucket",
     "photovogue",
+    "picarto",
     "piczel",
     "pillowfort",
     "pinterest",
@@ -107,28 +112,33 @@ modules = [
     "readcomiconline",
     "reddit",
     "redgifs",
+    "rule34us",
     "sankaku",
     "sankakucomplex",
     "seiga",
-    "seisoparty",
     "senmanga",
     "sexcom",
     "simplyhentai",
+    "skeb",
     "slickpic",
     "slideshare",
     "smugmug",
     "speakerdeck",
     "subscribestar",
     "tapas",
+    "telegraph",
+    "toyhouse",
     "tsumino",
     "tumblr",
     "tumblrgallery",
+    "twibooru",
     "twitter",
     "unsplash",
     "vanillarock",
     "vk",
     "vsco",
     "wallhaven",
+    "wallpapercave",
     "warosu",
     "weasyl",
     "webtoons",
@@ -143,12 +153,14 @@ modules = [
     "foolslide",
     "mastodon",
     "shopify",
+    "lolisafe",
     "imagehosts",
     "directlink",
     "recursive",
     "oauth",
     "test",
     "ytdl",
+    "generic",
 ]
 
 
