@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2014-2021 Mike Fährmann
+# Copyright 2014-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -38,6 +38,7 @@ class Extractor():
     request_interval = 0.0
     request_interval_min = 0.0
     request_timestamp = 0.0
+    tls12 = True
 
     def __init__(self, match):
         self.log = logging.getLogger(self.category)
@@ -54,6 +55,7 @@ class Extractor():
         self._retries = self.config("retries", 4)
         self._timeout = self.config("timeout", 30)
         self._verify = self.config("verify", True)
+        self._proxies = util.build_proxy_map(self.config("proxy"), self.log)
         self._interval = util.build_duration_func(
             self.config("sleep-request", self.request_interval),
             self.request_interval_min,
@@ -64,7 +66,6 @@ class Extractor():
 
         self._init_session()
         self._init_cookies()
-        self._init_proxies()
 
     @classmethod
     def from_url(cls, url):
@@ -103,10 +104,12 @@ class Extractor():
 
     def request(self, url, *, method="GET", session=None, retries=None,
                 encoding=None, fatal=True, notfound=None, **kwargs):
-        if retries is None:
-            retries = self._retries
         if session is None:
             session = self.session
+        if retries is None:
+            retries = self._retries
+        if "proxies" not in kwargs:
+            kwargs["proxies"] = self._proxies
         if "timeout" not in kwargs:
             kwargs["timeout"] = self._timeout
         if "verify" not in kwargs:
@@ -164,7 +167,8 @@ class Extractor():
             self.log.debug("%s (%s/%s)", msg, tries, retries+1)
             if tries > retries:
                 break
-            time.sleep(max(tries, self.request_interval))
+            time.sleep(
+                max(tries, self._interval()) if self._interval else tries)
             tries += 1
 
         raise exception.HttpError(msg, response)
@@ -179,7 +183,7 @@ class Extractor():
         elif until:
             if isinstance(until, datetime.datetime):
                 # convert to UTC timestamp
-                until = (until - util.EPOCH) / util.SECOND
+                until = util.datetime_to_timestamp(until)
             else:
                 until = float(until)
             seconds = until - now
@@ -218,6 +222,7 @@ class Extractor():
         self.session = session = requests.Session()
         headers = session.headers
         headers.clear()
+        ssl_options = ssl_ciphers = 0
 
         browser = self.config("browser") or self.browser
         if browser and isinstance(browser, str):
@@ -234,9 +239,20 @@ class Extractor():
                 platform = "Macintosh; Intel Mac OS X 11.5"
 
             if browser == "chrome":
-                _emulate_browser_chrome(session, platform)
+                if platform.startswith("Macintosh"):
+                    platform = platform.replace(".", "_") + "_2"
             else:
-                _emulate_browser_firefox(session, platform)
+                browser = "firefox"
+
+            for key, value in HTTP_HEADERS[browser]:
+                if value and "{}" in value:
+                    headers[key] = value.format(platform)
+                else:
+                    headers[key] = value
+
+            ssl_options |= (ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3 |
+                            ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1)
+            ssl_ciphers = SSL_CIPHERS[browser]
         else:
             headers["User-Agent"] = self.config("user-agent", (
                 "Mozilla/5.0 (Windows NT 10.0; Win64; x64; "
@@ -249,25 +265,31 @@ class Extractor():
         if custom_headers:
             headers.update(custom_headers)
 
-        ciphers = self.config("ciphers")
-        if ciphers:
-            if isinstance(ciphers, list):
-                ciphers = ":".join(ciphers)
-            session.mount("https://", HTTPSAdapter(ciphers))
-
-    def _init_proxies(self):
-        """Update the session's proxy map"""
-        proxies = self.config("proxy")
-        if proxies:
-            if isinstance(proxies, str):
-                proxies = {"http": proxies, "https": proxies}
-            if isinstance(proxies, dict):
-                for scheme, proxy in proxies.items():
-                    if "://" not in proxy:
-                        proxies[scheme] = "http://" + proxy.lstrip("/")
-                self.session.proxies = proxies
+        custom_ciphers = self.config("ciphers")
+        if custom_ciphers:
+            if isinstance(custom_ciphers, list):
+                ssl_ciphers = ":".join(custom_ciphers)
             else:
-                self.log.warning("invalid proxy specifier: %s", proxies)
+                ssl_ciphers = custom_ciphers
+
+        source_address = self.config("source-address")
+        if source_address:
+            if isinstance(source_address, str):
+                source_address = (source_address, 0)
+            else:
+                source_address = (source_address[0], source_address[1])
+
+        tls12 = self.config("tls12")
+        if tls12 is None:
+            tls12 = self.tls12
+        if not tls12:
+            ssl_options |= ssl.OP_NO_TLSv1_2
+            self.log.debug("TLS 1.2 disabled.")
+
+        adapter = _build_requests_adapter(
+            ssl_options, ssl_ciphers, source_address)
+        session.mount("https://", adapter)
+        session.mount("http://", adapter)
 
     def _init_cookies(self):
         """Populate the session's cookiejar"""
@@ -335,13 +357,26 @@ class Extractor():
         now = time.time()
 
         for cookie in self._cookiejar:
-            if cookie.name in names and cookie.domain == domain:
-                if cookie.expires and cookie.expires < now:
-                    self.log.warning("Cookie '%s' has expired", cookie.name)
-                else:
-                    names.discard(cookie.name)
-                    if not names:
-                        return True
+            if cookie.name in names and (
+                    not domain or cookie.domain == domain):
+
+                if cookie.expires:
+                    diff = int(cookie.expires - now)
+
+                    if diff <= 0:
+                        self.log.warning(
+                            "Cookie '%s' has expired", cookie.name)
+                        continue
+
+                    elif diff <= 86400:
+                        hours = diff // 3600
+                        self.log.warning(
+                            "Cookie '%s' will expire in less than %s hour%s",
+                            cookie.name, hours + 1, "s" if hours else "")
+
+                names.discard(cookie.name)
+                if not names:
+                    return True
         return False
 
     def _prepare_ddosguard_cookies(self):
@@ -570,7 +605,13 @@ class BaseExtractor(Extractor):
         if not self.category:
             for index, group in enumerate(match.groups()):
                 if group is not None:
-                    self.category, self.root = self.instances[index]
+                    if index:
+                        self.category, self.root = self.instances[index-1]
+                        if not self.root:
+                            self.root = text.root_from_url(match.group(0))
+                    else:
+                        self.root = group
+                        self.category = group.partition("://")[2]
                     break
         Extractor.__init__(self, match)
 
@@ -585,7 +626,9 @@ class BaseExtractor(Extractor):
         pattern_list = []
         instance_list = cls.instances = []
         for category, info in instances.items():
-            root = info["root"].rstrip("/")
+            root = info["root"]
+            if root:
+                root = root.rstrip("/")
             instance_list.append((category, root))
 
             pattern = info.get("pattern")
@@ -593,41 +636,83 @@ class BaseExtractor(Extractor):
                 pattern = re.escape(root[root.index(":") + 3:])
             pattern_list.append(pattern + "()")
 
-        return r"(?:https?://)?(?:" + "|".join(pattern_list) + r")"
+        return (
+            r"(?:" + cls.basecategory + r":(https?://[^/?#]+)|"
+            r"(?:https?://)?(?:" + "|".join(pattern_list) + r"))"
+        )
 
 
-class HTTPSAdapter(HTTPAdapter):
+class RequestsAdapter(HTTPAdapter):
 
-    def __init__(self, ciphers):
-        context = self.ssl_context = ssl.create_default_context()
-        context.options |= (ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3 |
-                            ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1)
-        context.set_ecdh_curve("prime256v1")
-        context.set_ciphers(ciphers)
+    def __init__(self, ssl_context=None, source_address=None):
+        self.ssl_context = ssl_context
+        self.source_address = source_address
         HTTPAdapter.__init__(self)
 
     def init_poolmanager(self, *args, **kwargs):
         kwargs["ssl_context"] = self.ssl_context
+        kwargs["source_address"] = self.source_address
         return HTTPAdapter.init_poolmanager(self, *args, **kwargs)
 
     def proxy_manager_for(self, *args, **kwargs):
         kwargs["ssl_context"] = self.ssl_context
+        kwargs["source_address"] = self.source_address
         return HTTPAdapter.proxy_manager_for(self, *args, **kwargs)
 
 
-def _emulate_browser_firefox(session, platform):
-    headers = session.headers
-    headers["User-Agent"] = ("Mozilla/5.0 (" + platform + "; rv:91.0) "
-                             "Gecko/20100101 Firefox/91.0")
-    headers["Accept"] = ("text/html,application/xhtml+xml,"
-                         "application/xml;q=0.9,image/webp,*/*;q=0.8")
-    headers["Accept-Language"] = "en-US,en;q=0.5"
-    headers["Accept-Encoding"] = "gzip, deflate"
-    headers["Referer"] = None
-    headers["Upgrade-Insecure-Requests"] = "1"
-    headers["Cookie"] = None
+def _build_requests_adapter(ssl_options, ssl_ciphers, source_address):
+    key = (ssl_options, ssl_ciphers, source_address)
+    try:
+        return _adapter_cache[key]
+    except KeyError:
+        pass
 
-    session.mount("https://", HTTPSAdapter(
+    if ssl_options or ssl_ciphers:
+        ssl_context = ssl.create_default_context()
+        if ssl_options:
+            ssl_context.options |= ssl_options
+        if ssl_ciphers:
+            ssl_context.set_ecdh_curve("prime256v1")
+            ssl_context.set_ciphers(ssl_ciphers)
+    else:
+        ssl_context = None
+
+    adapter = _adapter_cache[key] = RequestsAdapter(
+        ssl_context, source_address)
+    return adapter
+
+
+_adapter_cache = {}
+
+
+HTTP_HEADERS = {
+    "firefox": (
+        ("User-Agent", "Mozilla/5.0 ({}; rv:91.0) "
+                       "Gecko/20100101 Firefox/91.0"),
+        ("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,"
+                   "image/avif,*/*;q=0.8"),
+        ("Accept-Language", "en-US,en;q=0.5"),
+        ("Accept-Encoding", "gzip, deflate"),
+        ("Referer", None),
+        ("Connection", "keep-alive"),
+        ("Upgrade-Insecure-Requests", "1"),
+        ("Cookie", None),
+    ),
+    "chrome": (
+        ("Upgrade-Insecure-Requests", "1"),
+        ("User-Agent", "Mozilla/5.0 ({}) AppleWebKit/537.36 (KHTML, "
+                       "like Gecko) Chrome/92.0.4515.131 Safari/537.36"),
+        ("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,"
+                   "image/webp,image/apng,*/*;q=0.8"),
+        ("Referer", None),
+        ("Accept-Encoding", "gzip, deflate"),
+        ("Accept-Language", "en-US,en;q=0.9"),
+        ("Cookie", None),
+    ),
+}
+
+SSL_CIPHERS = {
+    "firefox": (
         "TLS_AES_128_GCM_SHA256:"
         "TLS_CHACHA20_POLY1305_SHA256:"
         "TLS_AES_256_GCM_SHA384:"
@@ -641,31 +726,13 @@ def _emulate_browser_firefox(session, platform):
         "ECDHE-ECDSA-AES128-SHA:"
         "ECDHE-RSA-AES128-SHA:"
         "ECDHE-RSA-AES256-SHA:"
-        "DHE-RSA-AES128-SHA:"
-        "DHE-RSA-AES256-SHA:"
+        "AES128-GCM-SHA256:"
+        "AES256-GCM-SHA384:"
         "AES128-SHA:"
         "AES256-SHA:"
         "DES-CBC3-SHA"
-    ))
-
-
-def _emulate_browser_chrome(session, platform):
-    if platform.startswith("Macintosh"):
-        platform = platform.replace(".", "_") + "_2"
-
-    headers = session.headers
-    headers["Upgrade-Insecure-Requests"] = "1"
-    headers["User-Agent"] = (
-        "Mozilla/5.0 (" + platform + ") AppleWebKit/537.36 "
-        "(KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36")
-    headers["Accept"] = ("text/html,application/xhtml+xml,application/xml;"
-                         "q=0.9,image/webp,image/apng,*/*;q=0.8")
-    headers["Referer"] = None
-    headers["Accept-Encoding"] = "gzip, deflate"
-    headers["Accept-Language"] = "en-US,en;q=0.9"
-    headers["Cookie"] = None
-
-    session.mount("https://", HTTPSAdapter(
+    ),
+    "chrome": (
         "TLS_AES_128_GCM_SHA256:"
         "TLS_AES_256_GCM_SHA384:"
         "TLS_CHACHA20_POLY1305_SHA256:"
@@ -682,7 +749,8 @@ def _emulate_browser_chrome(session, platform):
         "AES128-SHA:"
         "AES256-SHA:"
         "DES-CBC3-SHA"
-    ))
+    ),
+}
 
 
 # Undo automatic pyOpenSSL injection by requests
