@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -14,6 +14,7 @@ from ..cache import cache, memcache
 import collections
 import itertools
 import mimetypes
+import binascii
 import time
 import re
 
@@ -39,7 +40,6 @@ class DeviantartExtractor(Extractor):
         self.offset = 0
         self.flat = self.config("flat", True)
         self.extra = self.config("extra", False)
-        self.quality = self.config("quality", "100")
         self.original = self.config("original", True)
         self.comments = self.config("comments", False)
         self.user = match.group(1) or match.group(2)
@@ -53,9 +53,6 @@ class DeviantartExtractor(Extractor):
         else:
             self.unwatch = None
 
-        if self.quality:
-            self.quality = ",q_{}".format(self.quality)
-
         if self.original != "image":
             self._update_content = self._update_content_default
         else:
@@ -104,19 +101,8 @@ class DeviantartExtractor(Extractor):
 
                 if self.original and deviation["is_downloadable"]:
                     self._update_content(deviation, content)
-
-                if content["src"].startswith("https://images-wixmp-"):
-                    if deviation["index"] <= 790677560:
-                        # https://github.com/r888888888/danbooru/issues/4069
-                        intermediary, count = re.subn(
-                            r"(/f/[^/]+/[^/]+)/v\d+/.*",
-                            r"/intermediary\1", content["src"], 1)
-                        if count:
-                            deviation["_fallback"] = (content["src"],)
-                            content["src"] = intermediary
-                    if self.quality:
-                        content["src"] = re.sub(
-                            r",q_\d+", self.quality, content["src"], 1)
+                else:
+                    self._update_token(deviation, content)
 
                 yield self.commit(deviation, content)
 
@@ -151,11 +137,12 @@ class DeviantartExtractor(Extractor):
 
     def prepare(self, deviation):
         """Adjust the contents of a Deviation-object"""
-        try:
-            deviation["index"] = text.parse_int(
-                deviation["url"].rpartition("-")[2])
-        except KeyError:
-            deviation["index"] = 0
+        if "index" not in deviation:
+            try:
+                deviation["index"] = text.parse_int(
+                    deviation["url"].rpartition("-")[2])
+            except KeyError:
+                deviation["index"] = 0
 
         if self.user:
             deviation["username"] = self.user
@@ -302,6 +289,32 @@ class DeviantartExtractor(Extractor):
         if mtype and mtype.startswith("image/"):
             content.update(data)
 
+    def _update_token(self, deviation, content):
+        """Replace JWT to be able to remove width/height limits
+
+        All credit goes to @Ironchest337
+        for discovering and implementing this method
+        """
+        url, sep, _ = content["src"].partition("/v1/")
+        if not sep:
+            return
+
+        #  header = b'{"typ":"JWT","alg":"none"}'
+        payload = (
+            b'{"sub":"urn:app:","iss":"urn:app:","obj":[[{"path":"/f/' +
+            url.partition("/f/")[2].encode() +
+            b'"}]],"aud":["urn:service:file.download"]}'
+        )
+
+        deviation["_fallback"] = (content["src"],)
+        content["src"] = (
+            "{}?token=eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0.{}.".format(
+                url,
+                #  base64 of 'header' is precomputed as 'eyJ0eX...'
+                #  binascii.a2b_base64(header).rstrip(b"=\n").decode(),
+                binascii.b2a_base64(payload).rstrip(b"=\n").decode())
+        )
+
     def _limited_request(self, url, **kwargs):
         """Limits HTTP requests to one every 2 seconds"""
         kwargs["fatal"] = None
@@ -590,7 +603,10 @@ class DeviantartStashExtractor(DeviantartExtractor):
         if stash_id[0] == "0":
             uuid = text.extract(page, '//deviation/', '"')[0]
             if uuid:
-                yield self.api.deviation(uuid)
+                deviation = self.api.deviation(uuid)
+                deviation["index"] = text.parse_int(text.extract(
+                    page, 'gmi-deviationid="', '"')[0])
+                yield deviation
                 return
 
         for item in text.extract_iter(
@@ -746,32 +762,33 @@ class DeviantartPopularExtractor(DeviantartExtractor):
 
     def __init__(self, match):
         DeviantartExtractor.__init__(self, match)
-        self.search_term = self.time_range = self.category_path = None
         self.user = ""
 
         trange1, path, trange2, query = match.groups()
-        trange = trange1 or trange2
         query = text.parse_query(query)
+        self.search_term = query.get("q")
 
-        if not trange:
-            trange = query.get("order")
-
-        if path:
-            self.category_path = path.strip("/")
-        if trange:
-            if trange.startswith("popular-"):
-                trange = trange[8:]
-            self.time_range = trange.replace("-", "").replace("hours", "hr")
-        if query:
-            self.search_term = query.get("q")
+        trange = trange1 or trange2 or query.get("order", "")
+        if trange.startswith("popular-"):
+            trange = trange[8:]
+        self.time_range = {
+            "newest"      : "now",
+            "most-recent" : "now",
+            "this-week"   : "1week",
+            "this-month"  : "1month",
+            "this-century": "alltime",
+            "all-time"    : "alltime",
+        }.get(trange, "alltime")
 
         self.popular = {
             "search": self.search_term or "",
-            "range" : trange or "",
-            "path"  : self.category_path,
+            "range" : trange or "all-time",
+            "path"  : path.strip("/") if path else "",
         }
 
     def deviations(self):
+        if self.time_range == "now":
+            return self.api.browse_newest(self.search_term, self.offset)
         return self.api.browse_popular(
             self.search_term, self.time_range, self.offset)
 
@@ -851,12 +868,8 @@ class DeviantartDeviationExtractor(DeviantartExtractor):
         }),
         # wixmp URL rewrite
         (("https://www.deviantart.com/citizenfresh/art/Hverarond-789295466"), {
-            "pattern": (r"https://images-wixmp-\w+\.wixmp\.com"
-                        r"/intermediary/f/[^/]+/[^.]+\.jpg")
-        }),
-        # wixmp URL rewrite v2 (#369)
-        (("https://www.deviantart.com/josephbiwald/art/Destiny-2-804940104"), {
-            "pattern": r"https://images-wixmp-\w+\.wixmp\.com/.*,q_100,"
+            "pattern": (r"https://images-wixmp-\w+\.wixmp\.com/f"
+                        r"/[^/]+/[^.]+\.jpg\?token="),
         }),
         # GIF (#242)
         (("https://www.deviantart.com/skatergators/art/COM-Moni-781571783"), {
@@ -991,6 +1004,7 @@ class DeviantartOAuthAPI():
         self.extractor = extractor
         self.log = extractor.log
         self.headers = {"dA-minor-version": "20200519"}
+        self._warn_429 = True
 
         self.delay = extractor.config("wait-min", 0)
         self.delay_min = max(2, self.delay)
@@ -1024,21 +1038,32 @@ class DeviantartOAuthAPI():
 
     def browse_deviantsyouwatch(self, offset=0):
         """Yield deviations from users you watch"""
-        endpoint = "browse/deviantsyouwatch"
+        endpoint = "/browse/deviantsyouwatch"
         params = {"limit": "50", "offset": offset,
                   "mature_content": self.mature}
         return self._pagination(endpoint, params, public=False)
 
     def browse_posts_deviantsyouwatch(self, offset=0):
         """Yield posts from users you watch"""
-        endpoint = "browse/posts/deviantsyouwatch"
+        endpoint = "/browse/posts/deviantsyouwatch"
         params = {"limit": "50", "offset": offset,
                   "mature_content": self.mature}
         return self._pagination(endpoint, params, public=False, unpack=True)
 
+    def browse_newest(self, query=None, offset=0):
+        """Browse newest deviations"""
+        endpoint = "/browse/newest"
+        params = {
+            "q"             : query,
+            "limit"         : 50 if self.metadata else 120,
+            "offset"        : offset,
+            "mature_content": self.mature,
+        }
+        return self._pagination(endpoint, params)
+
     def browse_popular(self, query=None, timerange=None, offset=0):
         """Yield popular deviations"""
-        endpoint = "browse/popular"
+        endpoint = "/browse/popular"
         params = {
             "q"             : query,
             "limit"         : 50 if self.metadata else 120,
@@ -1050,7 +1075,7 @@ class DeviantartOAuthAPI():
 
     def browse_tags(self, tag, offset=0):
         """ Browse a tag """
-        endpoint = "browse/tags"
+        endpoint = "/browse/tags"
         params = {
             "tag"           : tag,
             "offset"        : offset,
@@ -1061,14 +1086,14 @@ class DeviantartOAuthAPI():
 
     def browse_user_journals(self, username, offset=0):
         """Yield all journal entries of a specific user"""
-        endpoint = "browse/user/journals"
+        endpoint = "/browse/user/journals"
         params = {"username": username, "offset": offset, "limit": 50,
                   "mature_content": self.mature, "featured": "false"}
         return self._pagination(endpoint, params)
 
     def collections(self, username, folder_id, offset=0):
         """Yield all Deviation-objects contained in a collection folder"""
-        endpoint = "collections/" + folder_id
+        endpoint = "/collections/" + folder_id
         params = {"username": username, "offset": offset, "limit": 24,
                   "mature_content": self.mature}
         return self._pagination(endpoint, params)
@@ -1076,21 +1101,21 @@ class DeviantartOAuthAPI():
     @memcache(keyarg=1)
     def collections_folders(self, username, offset=0):
         """Yield all collection folders of a specific user"""
-        endpoint = "collections/folders"
+        endpoint = "/collections/folders"
         params = {"username": username, "offset": offset, "limit": 50,
                   "mature_content": self.mature}
         return self._pagination_list(endpoint, params)
 
     def comments_deviation(self, deviation_id, offset=0):
         """Fetch comments posted on a deviation"""
-        endpoint = "comments/deviation/" + deviation_id
+        endpoint = "/comments/deviation/" + deviation_id
         params = {"maxdepth": "5", "offset": offset, "limit": 50,
                   "mature_content": self.mature}
         return self._pagination_list(endpoint, params=params, key="thread")
 
     def deviation(self, deviation_id, public=True):
         """Query and return info about a single Deviation"""
-        endpoint = "deviation/" + deviation_id
+        endpoint = "/deviation/" + deviation_id
         deviation = self._call(endpoint, public=public)
         if self.metadata:
             self._metadata((deviation,))
@@ -1100,13 +1125,13 @@ class DeviantartOAuthAPI():
 
     def deviation_content(self, deviation_id, public=False):
         """Get extended content of a single Deviation"""
-        endpoint = "deviation/content"
+        endpoint = "/deviation/content"
         params = {"deviationid": deviation_id}
         return self._call(endpoint, params=params, public=public)
 
     def deviation_download(self, deviation_id, public=True):
         """Get the original file download (if allowed)"""
-        endpoint = "deviation/download/" + deviation_id
+        endpoint = "/deviation/download/" + deviation_id
         params = {"mature_content": self.mature}
         return self._call(endpoint, params=params, public=public)
 
@@ -1114,7 +1139,7 @@ class DeviantartOAuthAPI():
         """ Fetch deviation metadata for a set of deviations"""
         if not deviations:
             return []
-        endpoint = "deviation/metadata?" + "&".join(
+        endpoint = "/deviation/metadata?" + "&".join(
             "deviationids[{}]={}".format(num, deviation["deviationid"])
             for num, deviation in enumerate(deviations)
         )
@@ -1123,14 +1148,14 @@ class DeviantartOAuthAPI():
 
     def gallery(self, username, folder_id, offset=0, extend=True, public=True):
         """Yield all Deviation-objects contained in a gallery folder"""
-        endpoint = "gallery/" + folder_id
+        endpoint = "/gallery/" + folder_id
         params = {"username": username, "offset": offset, "limit": 24,
                   "mature_content": self.mature, "mode": "newest"}
         return self._pagination(endpoint, params, extend, public)
 
     def gallery_all(self, username, offset=0):
         """Yield all Deviation-objects of a specific user"""
-        endpoint = "gallery/all"
+        endpoint = "/gallery/all"
         params = {"username": username, "offset": offset, "limit": 24,
                   "mature_content": self.mature}
         return self._pagination(endpoint, params)
@@ -1138,7 +1163,7 @@ class DeviantartOAuthAPI():
     @memcache(keyarg=1)
     def gallery_folders(self, username, offset=0):
         """Yield all gallery folders of a specific user"""
-        endpoint = "gallery/folders"
+        endpoint = "/gallery/folders"
         params = {"username": username, "offset": offset, "limit": 50,
                   "mature_content": self.mature}
         return self._pagination_list(endpoint, params)
@@ -1146,12 +1171,12 @@ class DeviantartOAuthAPI():
     @memcache(keyarg=1)
     def user_profile(self, username):
         """Get user profile information"""
-        endpoint = "user/profile/" + username
+        endpoint = "/user/profile/" + username
         return self._call(endpoint, fatal=False)
 
     def user_friends_watch(self, username):
         """Watch a user"""
-        endpoint = "user/friends/watch/" + username
+        endpoint = "/user/friends/watch/" + username
         data = {
             "watch[friend]"       : "0",
             "watch[deviations]"   : "0",
@@ -1169,7 +1194,7 @@ class DeviantartOAuthAPI():
 
     def user_friends_unwatch(self, username):
         """Unwatch a user"""
-        endpoint = "user/friends/unwatch/" + username
+        endpoint = "/user/friends/unwatch/" + username
         return self._call(
             endpoint, method="POST", public=False, fatal=False,
         ).get("success")
@@ -1207,7 +1232,7 @@ class DeviantartOAuthAPI():
 
     def _call(self, endpoint, fatal=True, public=True, **kwargs):
         """Call an API endpoint"""
-        url = "https://www.deviantart.com/api/v1/oauth2/" + endpoint
+        url = "https://www.deviantart.com/api/v1/oauth2" + endpoint
         kwargs["fatal"] = None
 
         while True:
@@ -1236,6 +1261,16 @@ class DeviantartOAuthAPI():
                 if self.delay < 30:
                     self.delay += 1
                 self.log.warning("%s. Using %ds delay.", msg, self.delay)
+
+                if self._warn_429 and self.delay >= 3:
+                    self._warn_429 = False
+                    if self.client_id == self.CLIENT_ID:
+                        self.log.info(
+                            "Register your own OAuth application and use its "
+                            "credentials to prevent this error: "
+                            "https://github.com/mikf/gallery-dl/blob/master/do"
+                            "cs/configuration.rst#extractordeviantartclient-id"
+                            "--client-secret")
             else:
                 self.log.error(msg)
                 return data
@@ -1347,7 +1382,7 @@ class DeviantartEclipseAPI():
         self.log = extractor.log
 
     def deviation_extended_fetch(self, deviation_id, user=None, kind=None):
-        endpoint = "da-browse/shared_api/deviation/extended_fetch"
+        endpoint = "/da-browse/shared_api/deviation/extended_fetch"
         params = {
             "deviationid"    : deviation_id,
             "username"       : user,
@@ -1357,7 +1392,7 @@ class DeviantartEclipseAPI():
         return self._call(endpoint, params)
 
     def gallery_scraps(self, user, offset=None):
-        endpoint = "da-user-profile/api/gallery/contents"
+        endpoint = "/da-user-profile/api/gallery/contents"
         params = {
             "username"     : user,
             "offset"       : offset,
@@ -1367,7 +1402,7 @@ class DeviantartEclipseAPI():
         return self._pagination(endpoint, params)
 
     def user_watching(self, user, offset=None):
-        endpoint = "da-user-profile/api/module/watching"
+        endpoint = "/da-user-profile/api/module/watching"
         params = {
             "username": user,
             "moduleid": self._module_id_watching(user),
@@ -1377,7 +1412,7 @@ class DeviantartEclipseAPI():
         return self._pagination(endpoint, params)
 
     def _call(self, endpoint, params=None):
-        url = "https://www.deviantart.com/_napi/" + endpoint
+        url = "https://www.deviantart.com/_napi" + endpoint
         headers = {"Referer": "https://www.deviantart.com/"}
 
         response = self.extractor._limited_request(
