@@ -56,7 +56,7 @@ class FlickrImageExtractor(FlickrExtractor):
     subcategory = "image"
     pattern = (r"(?:https?://)?(?:"
                r"(?:(?:www\.|m\.)?flickr\.com/photos/[^/]+/"
-               r"|[^.]+\.static\.?flickr\.com/(?:\d+/)+)(\d+)"
+               r"|[\w-]+\.static\.?flickr\.com/(?:\d+/)+)(\d+)"
                r"|flic\.kr/p/([A-Za-z1-9]+))")
     test = (
         ("https://www.flickr.com/photos/departingyyz/16089302239", {
