@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2018-2020 Leonardo Taccari
-# Copyright 2018-2021 Mike Fährmann
+# Copyright 2018-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -43,6 +43,7 @@ class InstagramExtractor(Extractor):
         self.login()
         data = self.metadata()
         videos = self.config("videos", True)
+        previews = self.config("previews", False)
         video_headers = {"User-Agent": "Mozilla/5.0"}
 
         for post in self.posts():
@@ -56,14 +57,18 @@ class InstagramExtractor(Extractor):
 
             yield Message.Directory, post
             for file in files:
-                url = file.get("video_url")
-                if not url:
-                    url = file["display_url"]
-                elif not videos:
-                    continue
-                else:
-                    file["_http_headers"] = video_headers
                 file.update(post)
+
+                url = file.get("video_url")
+                if url:
+                    if videos:
+                        file["_http_headers"] = video_headers
+                        text.nameext_from_url(url, file)
+                        yield Message.Url, url, file
+                    if not previews:
+                        continue
+
+                url = file["display_url"]
                 yield Message.Url, url, text.nameext_from_url(url, file)
 
     def metadata(self):
@@ -174,10 +179,16 @@ class InstagramExtractor(Extractor):
         if post.get("is_video") and "video_url" not in post:
             url = "{}/tv/{}/".format(self.root, post["shortcode"])
             post = self._extract_post_page(url)
+            if "items" in post:
+                return self._parse_post_api({"media": post["items"][0]})
+            post = post["graphql"]["shortcode_media"]
         elif typename == "GraphSidecar" and \
                 "edge_sidecar_to_children" not in post:
             url = "{}/p/{}/".format(self.root, post["shortcode"])
             post = self._extract_post_page(url)
+            if "items" in post:
+                return self._parse_post_api({"media": post["items"][0]})
+            post = post["graphql"]["shortcode_media"]
 
         owner = post["owner"]
         data = {
@@ -347,7 +358,7 @@ class InstagramExtractor(Extractor):
         data = self._extract_shared_data(url)["entry_data"]
         if "HttpErrorPage" in data:
             raise exception.NotFoundError("post")
-        return data["PostPage"][0]["graphql"]["shortcode_media"]
+        return data["PostPage"][0]
 
     def _get_edge_data(self, user, key):
         cursor = self.config("cursor")
@@ -426,7 +437,7 @@ class InstagramPostsExtractor(InstagramExtractor):
         url = "{}/{}/".format(self.root, self.item)
         user = self._extract_profile_page(url)
 
-        query_hash = "7ea6ae3cf6fb05e73fcbe1732b1d2a42"
+        query_hash = "8c2a529969ee035a5063f2fc8602a0fd"
         variables = {"id": user["id"], "first": 50}
         edge = self._get_edge_data(user, "edge_owner_to_timeline_media")
         return self._pagination_graphql(query_hash, variables, edge)
@@ -439,15 +450,27 @@ class InstagramTaggedExtractor(InstagramExtractor):
     test = ("https://www.instagram.com/instagram/tagged/", {
         "range": "1-16",
         "count": ">= 16",
+        "keyword": {
+            "tagged_owner_id" : "25025320",
+            "tagged_username" : "instagram",
+            "tagged_full_name": "Instagram",
+        },
     })
 
-    def posts(self):
+    def metadata(self):
         url = "{}/{}/".format(self.root, self.item)
-        user = self._extract_profile_page(url)
+        self.user = user = self._extract_profile_page(url)
 
+        return {
+            "tagged_owner_id" : user["id"],
+            "tagged_username" : user["username"],
+            "tagged_full_name": user["full_name"],
+        }
+
+    def posts(self):
         query_hash = "be13233562af2d229b008d2976b998b5"
-        variables = {"id": user["id"], "first": 50}
-        edge = self._get_edge_data(user, None)
+        variables = {"id": self.user["id"], "first": 50}
+        edge = self._get_edge_data(self.user, None)
         return self._pagination_graphql(query_hash, variables, edge)
 
 
@@ -552,7 +575,7 @@ class InstagramPostExtractor(InstagramExtractor):
     """Extractor for an Instagram post"""
     subcategory = "post"
     pattern = (r"(?:https?://)?(?:www\.)?instagram\.com"
-               r"/(?:p|tv|reel)/([^/?#]+)")
+               r"/(?:[^/?#]+/)?(?:p|tv|reel)/([^/?#]+)")
     test = (
         # GraphImage
         ("https://www.instagram.com/p/BqvsDleB3lV/", {
@@ -651,11 +674,14 @@ class InstagramPostExtractor(InstagramExtractor):
             }
         }),
 
+        # URL with username (#2085)
+        ("https://www.instagram.com/dm/p/CW042g7B9CY/"),
+
         ("https://www.instagram.com/reel/CDg_6Y1pxWu/"),
     )
 
     def posts(self):
-        query_hash = "1f950d414a6e11c98c556aa007b3157d"
+        query_hash = "2efa04f61586458cef44441f474eee7c"
         variables = {
             "shortcode"            : self.item,
             "child_comment_count"  : 3,
@@ -674,14 +700,15 @@ class InstagramStoriesExtractor(InstagramExtractor):
     """Extractor for Instagram stories"""
     subcategory = "stories"
     pattern = (r"(?:https?://)?(?:www\.)?instagram\.com"
-               r"/stories/(?:highlights/(\d+)|([^/?#]+))")
+               r"/stories/(?:highlights/(\d+)|([^/?#]+)(?:/(\d+))?)")
     test = (
         ("https://www.instagram.com/stories/instagram/"),
         ("https://www.instagram.com/stories/highlights/18042509488170095/"),
+        ("https://instagram.com/stories/geekmig/2724343156064789461"),
     )
 
     def __init__(self, match):
-        self.highlight_id, self.user = match.groups()
+        self.highlight_id, self.user, self.media_id = match.groups()
         if self.highlight_id:
             self.subcategory = InstagramHighlightsExtractor.subcategory
         InstagramExtractor.__init__(self, match)
@@ -700,7 +727,18 @@ class InstagramStoriesExtractor(InstagramExtractor):
 
         endpoint = "/v1/feed/reels_media/"
         params = {"reel_ids": reel_id}
-        return self._request_api(endpoint, params=params)["reels"].values()
+        reels = self._request_api(endpoint, params=params)["reels"]
+
+        if self.media_id:
+            reel = reels[reel_id]
+            for item in reel["items"]:
+                if item["pk"] == self.media_id:
+                    reel["items"] = (item,)
+                    break
+            else:
+                raise exception.NotFoundError("story")
+
+        return reels.values()
 
 
 class InstagramHighlightsExtractor(InstagramExtractor):
@@ -713,26 +751,21 @@ class InstagramHighlightsExtractor(InstagramExtractor):
         url = "{}/{}/".format(self.root, self.item)
         user = self._extract_profile_page(url)
 
-        query_hash = "d4d88dc1500312af6f937f7b804c68c3"
-        variables = {
-            "user_id": user["id"],
-            "include_chaining": False,
-            "include_reel": True,
-            "include_suggested_users": False,
-            "include_logged_out_extras": False,
-            "include_highlight_reels": True,
-            "include_live_status": True,
-        }
-        data = self._request_graphql(query_hash, variables)
-        edges = data["user"]["edge_highlight_reels"]["edges"]
-        if not edges:
-            return ()
+        endpoint = "/v1/highlights/{}/highlights_tray/".format(user["id"])
+        tray = self._request_api(endpoint)["tray"]
+        reel_ids = [highlight["id"] for highlight in tray]
 
-        reel_ids = ["highlight:" + edge["node"]["id"] for edge in edges]
-        endpoint = "/v1/feed/reels_media/?reel_ids=" + \
-                   "&reel_ids=".join(text.quote(rid) for rid in reel_ids)
-        reels = self._request_api(endpoint)["reels"]
-        return [reels[rid] for rid in reel_ids]
+        # Anything above 30 responds with statuscode 400.
+        # 30 can work, however, sometimes the API will respond with 560 or 500.
+        chunk_size = 5
+        endpoint = "/v1/feed/reels_media/"
+
+        for offset in range(0, len(reel_ids), chunk_size):
+            chunk_ids = reel_ids[offset : offset+chunk_size]
+            params = {"reel_ids": chunk_ids}
+            reels = self._request_api(endpoint, params=params)["reels"]
+            for reel_id in chunk_ids:
+                yield reels[reel_id]
 
 
 class InstagramReelsExtractor(InstagramExtractor):
