@@ -6,7 +6,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extractors for mastodon instances"""
+"""Extractors for Mastodon instances"""
 
 from .common import BaseExtractor, Message
 from .. import text, exception
@@ -25,30 +25,37 @@ class MastodonExtractor(BaseExtractor):
         BaseExtractor.__init__(self, match)
         self.instance = self.root.partition("://")[2]
         self.item = match.group(match.lastindex)
+        self.reblogs = self.config("reblogs", False)
+        self.replies = self.config("replies", True)
 
     def items(self):
         for status in self.statuses():
+
+            if not self.reblogs and status["reblog"]:
+                self.log.debug("Skipping %s (reblog)", status["id"])
+                continue
+            if not self.replies and status["in_reply_to_id"]:
+                self.log.debug("Skipping %s (reply)", status["id"])
+                continue
+
             attachments = status["media_attachments"]
-            if attachments:
-                self.prepare(status)
-                yield Message.Directory, status
-                for media in attachments:
-                    status["media"] = media
-                    url = media["url"]
-                    yield Message.Url, url, text.nameext_from_url(url, status)
+            del status["media_attachments"]
+
+            status["instance"] = self.instance
+            status["tags"] = [tag["name"] for tag in status["tags"]]
+            status["date"] = text.parse_datetime(
+                status["created_at"][:19], "%Y-%m-%dT%H:%M:%S")
+
+            yield Message.Directory, status
+            for media in attachments:
+                status["media"] = media
+                url = media["url"]
+                yield Message.Url, url, text.nameext_from_url(url, status)
 
     def statuses(self):
-        """Return an iterable containing all relevant Status-objects"""
+        """Return an iterable containing all relevant Status objects"""
         return ()
 
-    def prepare(self, status):
-        """Prepare a status object"""
-        del status["media_attachments"]
-        status["instance"] = self.instance
-        status["tags"] = [tag["name"] for tag in status["tags"]]
-        status["date"] = text.parse_datetime(
-            status["created_at"][:19], "%Y-%m-%dT%H:%M:%S")
-
 
 INSTANCES = {
     "mastodon.social": {
@@ -80,7 +87,7 @@ BASE_PATTERN = MastodonExtractor.update(INSTANCES)
 class MastodonUserExtractor(MastodonExtractor):
     """Extractor for all images of an account/user"""
     subcategory = "user"
-    pattern = BASE_PATTERN + r"/@([^/?#]+)(?:/media)?/?$"
+    pattern = BASE_PATTERN + r"/(?:@|users/)([^/?#]+)(?:/media)?/?$"
     test = (
         ("https://mastodon.social/@jk", {
             "pattern": r"https://files.mastodon.social/media_attachments"
@@ -93,18 +100,42 @@ class MastodonUserExtractor(MastodonExtractor):
             "count": 60,
         }),
         ("https://baraag.net/@pumpkinnsfw"),
+        ("https://mastodon.social/@id:10843"),
+        ("https://mastodon.social/users/id:10843"),
+        ("https://mastodon.social/users/jk"),
     )
 
     def statuses(self):
         api = MastodonAPI(self)
-        username = self.item
-        handle = "@{}@{}".format(username, self.instance)
-        for account in api.account_search(handle, 1):
-            if account["username"] == username:
-                break
-        else:
-            raise exception.NotFoundError("account")
-        return api.account_statuses(account["id"])
+
+        return api.account_statuses(
+            api.account_id_by_username(self.item),
+            only_media=not self.config("text-posts", False),
+            exclude_replies=not self.replies,
+        )
+
+
+class MastodonFollowingExtractor(MastodonExtractor):
+    """Extractor for followed mastodon users"""
+    subcategory = "following"
+    pattern = BASE_PATTERN + r"/users/([^/?#]+)/following"
+    test = (
+        ("https://mastodon.social/users/0x4f/following", {
+            "extractor": False,
+            "count": ">= 20",
+        }),
+        ("https://mastodon.social/users/id:10843/following"),
+        ("https://pawoo.net/users/yoru_nine/following"),
+        ("https://baraag.net/users/pumpkinnsfw/following"),
+    )
+
+    def items(self):
+        api = MastodonAPI(self)
+        account_id = api.account_id_by_username(self.item)
+
+        for account in api.account_following(account_id):
+            account["_extractor"] = MastodonUserExtractor
+            yield Message.Queue, account["url"], account
 
 
 class MastodonStatusExtractor(MastodonExtractor):
@@ -130,8 +161,8 @@ class MastodonStatusExtractor(MastodonExtractor):
 class MastodonAPI():
     """Minimal interface for the Mastodon API
 
+    https://docs.joinmastodon.org/
     https://github.com/tootsuite/mastodon
-    https://github.com/tootsuite/documentation/blob/master/Using-the-API/API.md
     """
 
     def __init__(self, extractor):
@@ -152,16 +183,32 @@ class MastodonAPI():
 
         self.headers = {"Authorization": "Bearer " + access_token}
 
+    def account_id_by_username(self, username):
+        if username.startswith("id:"):
+            return username[3:]
+
+        handle = "@{}@{}".format(username, self.extractor.instance)
+        for account in self.account_search(handle, 1):
+            if account["username"] == username:
+                return account["id"]
+        raise exception.NotFoundError("account")
+
+    def account_following(self, account_id):
+        endpoint = "/v1/accounts/{}/following".format(account_id)
+        return self._pagination(endpoint, None)
+
     def account_search(self, query, limit=40):
-        """Search for content"""
+        """Search for accounts"""
         endpoint = "/v1/accounts/search"
         params = {"q": query, "limit": limit}
         return self._call(endpoint, params).json()
 
-    def account_statuses(self, account_id):
-        """Get an account's statuses"""
+    def account_statuses(self, account_id, only_media=True,
+                         exclude_replies=False):
+        """Fetch an account's statuses"""
         endpoint = "/v1/accounts/{}/statuses".format(account_id)
-        params = {"only_media": "1"}
+        params = {"only_media"     : "1" if only_media else "0",
+                  "exclude_replies": "1" if exclude_replies else "0"}
         return self._pagination(endpoint, params)
 
     def status(self, status_id):
@@ -202,6 +249,7 @@ class MastodonAPI():
             if not url:
                 return
             url = url["url"]
+            params = None
 
 
 @cache(maxage=100*365*24*3600, keyarg=0)
