@@ -46,7 +46,7 @@ class PhilomenaExtractor(BooruExtractor):
             try:
                 params["filter_id"] = INSTANCES[self.category]["filter_id"]
             except (KeyError, TypeError):
-                pass
+                params["filter_id"] = "2"
 
         while True:
             data = self.request(url, params=params).json()
@@ -62,6 +62,8 @@ INSTANCES = {
                    "filter_id": "56027"},
     "ponybooru" : {"root": "https://ponybooru.org",
                    "filter_id": "2"},
+    "furbooru"  : {"root": "https://furbooru.org",
+                   "filter_id": "2"},
 }
 
 BASE_PATTERN = PhilomenaExtractor.update(INSTANCES)
@@ -107,11 +109,11 @@ class PhilomenaPostExtractor(PhilomenaExtractor):
                 "source_url": "https://www.deviantart.com/speccysy/art"
                               "/Afternoon-Flight-215193985",
                 "spoilered": False,
-                "tag_count": 39,
+                "tag_count": 42,
                 "tag_ids": list,
                 "tags": list,
                 "thumbnails_generated": True,
-                "updated_at": "2021-07-13T14:22:40Z",
+                "updated_at": "2021-09-30T20:04:01Z",
                 "uploader": "Clover the Clever",
                 "uploader_id": 211188,
                 "upvotes": int,
@@ -124,6 +126,9 @@ class PhilomenaPostExtractor(PhilomenaExtractor):
         ("https://ponybooru.org/images/1", {
             "content": "bca26f58fafd791fe07adcd2a28efd7751824605",
         }),
+        ("https://furbooru.org/images/1", {
+            "content": "9eaa1e1b32fa0f16520912257dbefaff238d5fd2",
+        }),
     )
 
     def __init__(self, match):
@@ -157,13 +162,17 @@ class PhilomenaSearchExtractor(PhilomenaExtractor):
             "range": "40-60",
             "count": 21,
         }),
+        ("https://furbooru.org/search?q=cute", {
+            "range": "40-60",
+            "count": 21,
+        }),
     )
 
     def __init__(self, match):
         PhilomenaExtractor.__init__(self, match)
         groups = match.groups()
         if groups[-1]:
-            q = groups[-1]
+            q = groups[-1].replace("+", " ")
             for old, new in (
                 ("-colon-"  , ":"),
                 ("-dash-"   , "-"),
@@ -210,6 +219,9 @@ class PhilomenaGalleryExtractor(PhilomenaExtractor):
         ("https://ponybooru.org/galleries/27", {
             "count": ">= 24",
         }),
+        ("https://furbooru.org/galleries/27", {
+            "count": ">= 13",
+        }),
     )
 
     def __init__(self, match):
