@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2021 Mike Fährmann
+# Copyright 2017-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -19,7 +19,7 @@ class RedditExtractor(Extractor):
     directory_fmt = ("{category}", "{subreddit}")
     filename_fmt = "{id}{num:? //>02} {title[:220]}.{extension}"
     archive_fmt = "{filename}"
-    cookiedomain = None
+    cookiedomain = ".reddit.com"
 
     def items(self):
         self.api = RedditAPI(self)
@@ -112,12 +112,18 @@ class RedditExtractor(Extractor):
         """Return an iterable containing all (submission, comments) tuples"""
 
     def _extract_gallery(self, submission):
-        if submission["gallery_data"] is None:
+        gallery = submission["gallery_data"]
+        if gallery is None:
             self.log.warning("gallery %s: deleted", submission["id"])
             return
 
-        meta = submission["media_metadata"]
-        for item in submission["gallery_data"]["items"]:
+        meta = submission.get("media_metadata")
+        if meta is None:
+            self.log.warning("gallery %s: missing 'media_metadata'",
+                             submission["id"])
+            return
+
+        for item in gallery["items"]:
             data = meta[item["media_id"]]
             if data["status"] != "valid" or "s" not in data:
                 self.log.warning(
@@ -166,7 +172,7 @@ class RedditUserExtractor(RedditExtractor):
     """Extractor for URLs from posts by a reddit user"""
     subcategory = "user"
     pattern = (r"(?:https?://)?(?:\w+\.)?reddit\.com/u(?:ser)?/"
-               r"([^/?#]+(?:/([a-z]+))?)/?(?:\?([^#]*))?")
+               r"([^/?#]+(?:/([a-z]+))?)/?(?:\?([^#]*))?$")
     test = (
         ("https://www.reddit.com/user/username/", {
             "count": ">= 2",
@@ -191,8 +197,8 @@ class RedditSubmissionExtractor(RedditExtractor):
     """Extractor for URLs from a submission on reddit.com"""
     subcategory = "submission"
     pattern = (r"(?:https?://)?(?:"
-               r"(?:\w+\.)?reddit\.com/(?:r/[^/?#]+/comments|gallery)"
-               r"|redd\.it)/([a-z0-9]+)")
+               r"(?:\w+\.)?reddit\.com/(?:(?:r|u|user)/[^/?#]+"
+               r"/comments|gallery)|redd\.it)/([a-z0-9]+)")
     test = (
         ("https://www.reddit.com/r/lavaporn/comments/8cqhub/", {
             "pattern": r"https://c2.staticflickr.com/8/7272/\w+_k.jpg",
@@ -221,10 +227,18 @@ class RedditSubmissionExtractor(RedditExtractor):
         ("https://www.reddit.com/r/cosplay/comments/jvwaqr", {
             "count": 1,
         }),
+        # gallery with no 'media_metadata' (#2001)
+        ("https://www.reddit.com/r/kpopfap/comments/qjj04q/", {
+            "count": 0,
+        }),
         ("https://old.reddit.com/r/lavaporn/comments/2a00np/"),
         ("https://np.reddit.com/r/lavaporn/comments/2a00np/"),
         ("https://m.reddit.com/r/lavaporn/comments/2a00np/"),
         ("https://redd.it/2a00np/"),
+        ("https://www.reddit.com/user/TheSpiritTree/comments/srilyf/", {
+            "pattern": r"https://i.redd.it/8fpgv17yqlh81.jpg",
+            "count": 1,
+        }),
     )
 
     def __init__(self, match):
@@ -291,6 +305,12 @@ class RedditAPI():
         else:
             self.refresh_token = token
 
+        if not self.refresh_token:
+            # allow downloading from quarantined subreddits (#2180)
+            extractor._cookiejar.set(
+                "_options", '%7B%22pref_quarantine_optin%22%3A%20true%7D',
+                domain=extractor.cookiedomain)
+
     def submission(self, submission_id):
         """Fetch the (submission, comments)=-tuple for a submission id"""
         endpoint = "/comments/" + submission_id + "/.json"
