@@ -78,8 +78,8 @@ class SankakuTagExtractor(SankakuExtractor):
     test = (
         ("https://sankaku.app/?tags=bonocho", {
             "count": 5,
-            "pattern": r"https://c?s\.sankakucomplex\.com/data/[^/]{2}/[^/]{2}"
-                       r"/[^/]{32}\.\w+\?e=\d+&m=[^&#]+",
+            "pattern": r"https://v\.sankakucomplex\.com/data/[^/]{2}/[^/]{2}"
+                       r"/[^/]{32}\.\w+\?e=\d+&expires=\d+&m=[^&#]+",
         }),
         ("https://beta.sankakucomplex.com/?tags=bonocho"),
         ("https://chan.sankakucomplex.com/?tags=bonocho"),
@@ -159,7 +159,7 @@ class SankakuPostExtractor(SankakuExtractor):
         }),
         # 'contentious_content'
         ("https://sankaku.app/post/show/21418978", {
-            "pattern": r"https://s\.sankakucomplex\.com"
+            "pattern": r"https://v\.sankakucomplex\.com"
                        r"/data/13/3c/133cda3bfde249c504284493903fb985\.jpg",
         }),
         # empty tags (#1617)
