@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2020 Mike Fährmann
+# Copyright 2019-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -21,7 +21,6 @@ class SexcomExtractor(Extractor):
     root = "https://www.sex.com"
 
     def items(self):
-        yield Message.Version, 1
         yield Message.Directory, self.metadata()
         for pin in map(self._parse_pin, self.pins()):
             if pin:
@@ -79,9 +78,14 @@ class SexcomExtractor(Extractor):
                     path += "/hd"
                 data["url"] = self.root + path
             else:
+                iframe = extr('<iframe', '>')
+                src = (text.extract(iframe, ' src="', '"')[0] or
+                       text.extract(iframe, " src='", "'")[0])
+                if not src:
+                    self.log.warning("Unable to fetch media from %s", url)
+                    return None
                 data["extension"] = None
-                data["url"] = "ytdl:" + text.extract(
-                    extr('<iframe', '>'), ' src="', '"')[0]
+                data["url"] = "ytdl:" + src
         else:
             data["url"] = text.unescape(extr(' src="', '"').partition("?")[0])
             text.nameext_from_url(data["url"], data)
@@ -163,6 +167,27 @@ class SexcomRelatedPinExtractor(SexcomPinExtractor):
         return self._pagination(url)
 
 
+class SexcomPinsExtractor(SexcomExtractor):
+    """Extractor for a user's pins on www.sex.com"""
+    subcategory = "pins"
+    directory_fmt = ("{category}", "{user}")
+    pattern = r"(?:https?://)?(?:www\.)?sex\.com/user/([^/?#]+)/pins/"
+    test = ("https://www.sex.com/user/sirjuan79/pins/", {
+        "count": ">= 15",
+    })
+
+    def __init__(self, match):
+        SexcomExtractor.__init__(self, match)
+        self.user = match.group(1)
+
+    def metadata(self):
+        return {"user": text.unquote(self.user)}
+
+    def pins(self):
+        url = "{}/user/{}/pins/".format(self.root, self.user)
+        return self._pagination(url)
+
+
 class SexcomBoardExtractor(SexcomExtractor):
     """Extractor for pins from a board on www.sex.com"""
     subcategory = "board"
@@ -193,7 +218,7 @@ class SexcomSearchExtractor(SexcomExtractor):
     subcategory = "search"
     directory_fmt = ("{category}", "search", "{search[query]}")
     pattern = (r"(?:https?://)?(?:www\.)?sex\.com/((?:"
-               r"(pic|gif|video)s/([^/?#]+)|search/(pic|gif|video)s"
+               r"(pic|gif|video)s/([^/?#]*)|search/(pic|gif|video)s"
                r")/?(?:\?([^#]+))?)")
     test = (
         ("https://www.sex.com/search/pics?query=ecchi", {
@@ -204,6 +229,7 @@ class SexcomSearchExtractor(SexcomExtractor):
             "range": "1-10",
             "count": 10,
         }),
+        ("https://www.sex.com/pics/?sort=popular&sub=all&page=1"),
     )
 
     def __init__(self, match):
