@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021 Mike Fährmann
+# Copyright 2021-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,14 +8,20 @@
 
 """String formatters"""
 
+import os
 import json
 import string
 import _string
+import datetime
 import operator
 from . import text, util
 
 _CACHE = {}
 _CONVERSIONS = None
+_GLOBALS = {
+    "_env": lambda: os.environ,
+    "_now": datetime.datetime.now,
+}
 
 
 def parse(format_string, default=None):
@@ -37,6 +43,8 @@ def parse(format_string, default=None):
             cls = ExpressionFormatter
         elif kind == "M":
             cls = ModuleFormatter
+        elif kind == "F":
+            cls = FStringFormatter
 
     formatter = _CACHE[key] = cls(format_string, default)
     return formatter
@@ -58,7 +66,7 @@ class StringFormatter():
     - "j". calls json.dumps
     - "t": calls str.strip
     - "d": calls text.parse_timestamp
-    - "U": calls urllib.parse.unquote
+    - "U": calls urllib.parse.unescape
     - "S": calls util.to_string()
     - "T": calls util.to_timestamü()
     - Example: {f!l} -> "example"; {f!u} -> "EXAMPLE"
@@ -125,6 +133,8 @@ class StringFormatter():
             ], fmt)
         else:
             key, funcs = parse_field_name(field_name)
+            if key in _GLOBALS:
+                return self._apply_globals(_GLOBALS[key], funcs, fmt)
             if funcs:
                 return self._apply(key, funcs, fmt)
             return self._apply_simple(key, fmt)
@@ -140,6 +150,17 @@ class StringFormatter():
             return fmt(obj)
         return wrap
 
+    def _apply_globals(self, gobj, funcs, fmt):
+        def wrap(_):
+            try:
+                obj = gobj()
+                for func in funcs:
+                    obj = func(obj)
+            except Exception:
+                obj = self.default
+            return fmt(obj)
+        return wrap
+
     def _apply_simple(self, key, fmt):
         def wrap(kwdict):
             return fmt(kwdict[key] if key in kwdict else self.default)
@@ -149,7 +170,7 @@ class StringFormatter():
         def wrap(kwdict):
             for key, funcs in lst:
                 try:
-                    obj = kwdict[key]
+                    obj = _GLOBALS[key]() if key in _GLOBALS else kwdict[key]
                     for func in funcs:
                         obj = func(obj)
                     if obj:
@@ -187,6 +208,13 @@ class ModuleFormatter():
         self.format_map = getattr(module, function_name)
 
 
+class FStringFormatter():
+    """Generate text by evaluaring an f-string literal"""
+
+    def __init__(self, fstring, default=None):
+        self.format_map = util.compile_expression("f'''" + fstring + "'''")
+
+
 def parse_field_name(field_name):
     first, rest = _string.formatter_field_name_split(field_name)
     funcs = []
@@ -226,7 +254,7 @@ def parse_format_spec(format_spec, conversion):
             "C": string.capwords,
             "j": json.dumps,
             "t": str.strip,
-            "T": util.to_timestamp,
+            "T": util.datetime_to_timestamp_string,
             "d": text.parse_timestamp,
             "U": text.unescape,
             "S": util.to_string,
@@ -255,6 +283,8 @@ def build_format_func(format_spec):
             return _parse_join(format_spec)
         if fmt == "R":
             return _parse_replace(format_spec)
+        if fmt == "D":
+            return _parse_datetime(format_spec)
         return _default_format(format_spec)
     return format
 
@@ -300,6 +330,16 @@ def _parse_replace(format_spec):
     return replace
 
 
+def _parse_datetime(format_spec):
+    dt_format, _, format_spec = format_spec.partition("/")
+    dt_format = dt_format[1:]
+    fmt = build_format_func(format_spec)
+
+    def dt(obj):
+        return fmt(text.parse_datetime(obj, dt_format))
+    return dt
+
+
 def _default_format(format_spec):
     def wrap(obj):
         return format(obj, format_spec)
